@@ -20,10 +20,18 @@
 #include <neo4j-client.h>
 #include "state.h"
 
-int auth_reattempt(void *userdata, const char *host, unsigned int attempts,
-        int error, char *username, size_t usize, char *password, size_t psize);
 
-int change_password(shell_state_t *state, neo4j_session_t *session,
+struct auth_state
+{
+    shell_state_t *state;
+    unsigned int attempt;
+};
+
+
+int basic_auth(struct auth_state *auth_state, const char *host,
+        char *username, size_t usize, char *password, size_t psize);
+
+int change_password(shell_state_t *state, neo4j_connection_t *connection,
         char *password, size_t pwlen);
 
 #endif/*NEO4J_AUTHENTICATION_H*/
