@@ -41,7 +41,7 @@ struct parse_callback_data
 
 static int parse_string(void *data, struct parse_callback_data *cdata);
 static int parse_stream(void *data, struct parse_callback_data *cdata);
-static int process(shell_state_t *state,
+static int process(shell_state_t *state, struct cypher_input_position pos,
         int (*parse)(void *data, struct parse_callback_data *cdata),
         void *parse_data);
 
@@ -54,27 +54,38 @@ static int finalize(shell_state_t *state, evaluation_queue_t *queue,
 static int abort_outstanding(shell_state_t *state, evaluation_queue_t *queue);
 
 
-int source(shell_state_t *state, const char *filename)
+int source(shell_state_t *state, struct cypher_input_position pos,
+        const char *filename)
 {
     if (state->source_depth >= state->source_max_depth)
     {
-        fprintf(state->err, "Too many nested calls to `:source`\n");
+        print_error(state, pos, "Too many nested calls to `:source`");
         return -1;
     }
 
-    FILE *stream = fopen(filename, "r");
-    if (stream == NULL)
+    FILE *stream;
+    if (strcmp(filename, "-") == 0)
     {
-        fprintf(state->err, "Unable to read file '%s': %s\n",
-                filename, strerror(errno));
-        return -1;
+        stream = stdin;
+        filename = "<stdin>";
     }
+    else
+    {
+        stream = fopen(filename, "r");
+        if (stream == NULL)
+        {
+            print_error(state, pos, "Unable to read file '%s': %s",
+                    filename, strerror(errno));
+            return -1;
+        }
+    }
+
     bool interactive = state->interactive;
     state->interactive = false;
     const char *prev_infile = state->infile;
     state->infile = filename;
     ++(state->source_depth);
-    int result = batch(state, stream);
+    int result = batch(state, pos, stream);
     fclose(stream);
     --(state->source_depth);
     state->infile = prev_infile;
@@ -87,15 +98,16 @@ int source(shell_state_t *state, const char *filename)
 }
 
 
-int eval(shell_state_t *state, const char *script)
+int eval(shell_state_t *state, struct cypher_input_position pos,
+        const char *script)
 {
-    return process(state, parse_string, (void *)(uintptr_t)script);
+    return process(state, pos, parse_string, (void *)(uintptr_t)script);
 }
 
 
-int batch(shell_state_t *state, FILE *stream)
+int batch(shell_state_t *state, struct cypher_input_position pos, FILE *stream)
 {
-    return process(state, parse_stream, (void *)stream);
+    return process(state, pos, parse_stream, (void *)stream);
 }
 
 
@@ -111,7 +123,7 @@ int parse_stream(void *data, struct parse_callback_data *cdata)
 }
 
 
-int process(shell_state_t *state,
+int process(shell_state_t *state, struct cypher_input_position pos,
         int (*parse)(void *data, struct parse_callback_data *cdata),
         void *parse_data)
 {
@@ -119,7 +131,7 @@ int process(shell_state_t *state,
             (state->pipeline_max * sizeof(evaluation_continuation_t *)));
     if (queue == NULL)
     {
-        neo4j_perror(state->err, errno, "unexpected error");
+        print_error_errno(state, pos, errno, "calloc");
         return -1;
     }
     queue->capacity = state->pipeline_max;
@@ -131,7 +143,7 @@ int process(shell_state_t *state,
     {
         if (err != -2)
         {
-            neo4j_perror(state->err, errno, "unexpected error");
+            print_errno(state, pos, errno);
         }
         goto cleanup;
     }
@@ -178,6 +190,9 @@ int evaluate(shell_state_t *state, evaluation_queue_t *queue,
         return 0;
     }
 
+    struct cypher_input_range range =
+            cypher_quick_parse_segment_get_range(segment);
+
     if (cypher_quick_parse_segment_is_command(segment))
     {
         // drain queue before running commands
@@ -186,25 +201,20 @@ int evaluate(shell_state_t *state, evaluation_queue_t *queue,
         {
             return err;
         }
-        return evaluate_command(state, s, n);
+        return evaluate_command(state, s, n, range.start);
     }
 
     assert(queue->depth <= queue->capacity);
     if ((queue->depth >= queue->capacity) && finalize(state, queue, 1))
     {
-        neo4j_perror(state->err, errno, "unexpected error");
         return -1;
     }
-    assert (queue->depth < queue->capacity);
-
-    struct cypher_input_range range =
-            cypher_quick_parse_segment_get_range(segment);
+    assert(queue->depth < queue->capacity);
 
     evaluation_continuation_t *continuation =
-            evaluate_statement(state, s, n, range.start);
+            prepare_statement(state, s, n, range.start);
     if (continuation == NULL)
     {
-        neo4j_perror(state->err, errno, "unexpected error");
         return -1;
     }
 
