@@ -19,8 +19,8 @@
 
 #include "state.h"
 
-int db_connect(shell_state_t *state, const char *connect_string,
-        const char *port_string);
-int db_disconnect(shell_state_t *state);
+int db_connect(shell_state_t *state, struct cypher_input_position pos,
+        const char *connect_string, const char *port_string);
+int db_disconnect(shell_state_t *state, struct cypher_input_position pos);
 
 #endif/*NEO4J_CONNECT_H*/
