@@ -19,11 +19,15 @@
 
 #include "state.h"
 
-int evaluate_command(shell_state_t *state, const char *command, size_t n);
+int evaluate_command(shell_state_t *state, const char *command, size_t n,
+        struct cypher_input_position pos);
+
+int evaluate_statement(shell_state_t *state, const char *statement, size_t n,
+        struct cypher_input_position pos);
 
 typedef struct evaluation_continuation evaluation_continuation_t;
 
-evaluation_continuation_t *evaluate_statement(shell_state_t *state,
+evaluation_continuation_t *prepare_statement(shell_state_t *state,
         const char *statement, size_t n, struct cypher_input_position pos);
 
 int complete_evaluation(evaluation_continuation_t *continuation,
@@ -32,4 +36,6 @@ int complete_evaluation(evaluation_continuation_t *continuation,
 int abort_evaluation(evaluation_continuation_t *continuation,
         shell_state_t *state);
 
+int display_schema(shell_state_t* state, struct cypher_input_position pos);
+
 #endif/*NEO4J_EVALUATE_H*/
