@@ -27,6 +27,7 @@
 #include <errno.h>
 #include <getopt.h>
 #include <limits.h>
+#include <locale.h>
 #include <neo4j-client.h>
 #include <signal.h>
 #include <stdio.h>
@@ -39,6 +40,8 @@
 
 
 #define NEO4J_HISTORY_FILE "client-history"
+#define NEO4J_USERNAME_ENV "NEO4J_USERNAME"
+#define NEO4J_PASSWORD_ENV "NEO4J_PASSWORD"
 
 
 const char *shortopts = "e:hi:o:p:Pu:v";
@@ -85,8 +88,8 @@ static struct option longopts[] =
 static void usage(FILE *s, const char *prog_name)
 {
     fprintf(s,
-"usage: %s [OPTIONS] URL\n"
-"       %s [OPTIONS] host [port]\n"
+"usage: %1$s [OPTIONS] [URL]\n"
+"       %1$s [OPTIONS] [host [port]]\n"
 "options:\n"
 " --help, -h          Output this usage information.\n"
 " --history=file      Use the specified file for saving history.\n"
@@ -97,11 +100,13 @@ static void usage(FILE *s, const char *prog_name)
 " --ca-directory=dir  Specify a directory containing trusted certificates.\n"
 " --insecure          Do not attempt to establish a secure connection.\n"
 " --non-interactive   Use non-interactive mode and do not prompt for\n"
-"                     credentials when connecting.\n"
+"                     host verification or credentials when connecting\n"
+"                     (default when no TTY is connected to the process).\n"
 " --username=name, -u name\n"
 "                     Connect using the specified username.\n"
 " --password=pass, -p pass\n"
-"                     Connect using the specified password.\n"
+"                     Connect using the specified password. This is only\n"
+"                     valid when a URL or host is also specified.\n"
 " -P                  Prompt for a password, even in non-interactive mode.\n"
 " --known-hosts=file  Set the path to the known-hosts file.\n"
 " --no-known-hosts    Do not do host checking via known-hosts (use only TLS\n"
@@ -111,10 +116,11 @@ static void usage(FILE *s, const char *prog_name)
 "                     specified in conjunction with --source/-i, and may be\n"
 "                     specified multiple times.\n"
 " --source file, -i file\n"
-"                     Read input from the specified file. May be specified\n"
-"                     multiple times.\n"
+"                     Evaluate statements from the specified input file.\n"
+"                     May be specified multiple times.\n"
 " --eval script, -e script\n"
-"                     Evaluate the script. May be specified multiple times\n"
+"                     Evaluate the argument string. May be specified multiple\n"
+"                     times.\n"
 " --verbose, -v       Increase logging verbosity.\n"
 " --version           Output the version of neo4j-client and dependencies.\n"
 "\n"
@@ -124,7 +130,7 @@ static void usage(FILE *s, const char *prog_name)
 "If the shell is run connected to a TTY, then an interactive command prompt\n"
 "is shown. Use `:exit` to quit. If the shell is not connected to a TTY, then\n"
 "directives are read from stdin.\n",
-        prog_name, prog_name);
+        prog_name);
 }
 
 static shell_state_t state;
@@ -133,7 +139,8 @@ struct io_handler
 {
     char *arg;
     bool is_output;
-    int (*handle)(shell_state_t *state, const char *arg);
+    int (*handle)(shell_state_t *state, struct cypher_input_position pos,
+            const char *arg);
 };
 
 #define NEO4J_MAX_IO_ARGS 128
@@ -142,22 +149,25 @@ static void interrupt_handler(int signal);
 static int add_io_handler(struct io_handler *io_handlers,
         unsigned int *nio_handlers,
         const char *arg, bool is_output,
-        int (*handler)(shell_state_t *state, const char *arg));
+        int (*handler)(shell_state_t *state, struct cypher_input_position pos,
+            const char *arg));
 
 
 int main(int argc, char *argv[])
 {
+    setlocale(LC_ALL,"");
+
     FILE *tty = fopen(_PATH_TTY, "r+");
     if (tty == NULL && errno != ENOENT)
     {
-        perror("can't open " _PATH_TTY);
+        perror("Can't open " _PATH_TTY);
         exit(EXIT_FAILURE);
     }
 
     char prog_name[PATH_MAX];
     if (neo4j_basename(argv[0], prog_name, sizeof(prog_name)) < 0)
     {
-        perror("unexpected error");
+        perror("Unexpected error");
         exit(EXIT_FAILURE);
     }
 
@@ -172,24 +182,50 @@ int main(int argc, char *argv[])
 
     if (shell_state_init(&state, prog_name, stdin, stdout, stderr, tty))
     {
-        neo4j_perror(state.err, errno, "unexpected error");
+        neo4j_perror(stderr, errno, "Unexpected error");
         goto cleanup;
     }
 
-    state.interactive = isatty(STDIN_FILENO);
+    state.interactive = isatty(STDIN_FILENO) && isatty(STDOUT_FILENO);
 
     char histfile[PATH_MAX];
     if (neo4j_dot_dir(histfile, sizeof(histfile), NEO4J_HISTORY_FILE) < 0)
     {
         neo4j_perror(state.err, (errno == ERANGE)? ENAMETOOLONG : errno,
-                "unexpected error");
+                "Unexpected error");
         goto cleanup;
     }
     state.histfile = histfile;
 
-    if (isatty(fileno(stderr)))
+    if (isatty(STDERR_FILENO))
     {
-        state.error_colorize = ansi_error_colorization;
+        state.colorize = ansi_shell_colorization;
+    }
+
+    if (isatty(STDOUT_FILENO))
+    {
+        neo4j_config_set_results_table_colors(state.config,
+                neo4j_results_table_ansi_colors);
+        neo4j_config_set_plan_table_colors(state.config,
+                neo4j_plan_table_ansi_colors);
+    }
+
+    const char *s;
+    if ((s = getenv(NEO4J_USERNAME_ENV)) != NULL &&
+            neo4j_config_set_username(state.config, s))
+    {
+        neo4j_perror(state.err, errno, "Unexpected error");
+        goto cleanup;
+    }
+    bool password_set = false;
+    if ((s = getenv(NEO4J_PASSWORD_ENV)) != NULL)
+    {
+        if (neo4j_config_set_password(state.config, s))
+        {
+            neo4j_perror(state.err, errno, "Unexpected error");
+            goto cleanup;
+        }
+        password_set = true;
     }
 
     int c;
@@ -210,22 +246,30 @@ int main(int argc, char *argv[])
         case CA_FILE_OPT:
             if (neo4j_config_set_TLS_ca_file(state.config, optarg))
             {
-                neo4j_perror(state.err, errno, "unexpected error");
+                neo4j_perror(state.err, errno, "Unexpected error");
                 goto cleanup;
             }
             break;
         case CA_DIRECTORY_OPT:
             if (neo4j_config_set_TLS_ca_dir(state.config, optarg))
             {
-                neo4j_perror(state.err, errno, "unexpected error");
+                neo4j_perror(state.err, errno, "Unexpected error");
                 goto cleanup;
             }
             break;
         case COLORIZE_OPT:
-            state.error_colorize = ansi_error_colorization;
+            state.colorize = ansi_shell_colorization;
+            neo4j_config_set_results_table_colors(state.config,
+                    neo4j_results_table_ansi_colors);
+            neo4j_config_set_plan_table_colors(state.config,
+                    neo4j_plan_table_ansi_colors);
             break;
         case NO_COLORIZE_OPT:
-            state.error_colorize = no_error_colorization;
+            state.colorize = no_shell_colorization;
+            neo4j_config_set_results_table_colors(state.config,
+                    neo4j_results_table_no_colors);
+            neo4j_config_set_plan_table_colors(state.config,
+                    neo4j_plan_table_no_colors);
             break;
         case INSECURE_OPT:
             state.connect_flags |= NEO4J_INSECURE;
@@ -241,37 +285,32 @@ int main(int argc, char *argv[])
         case 'u':
             if (neo4j_config_set_username(state.config, optarg))
             {
-                neo4j_perror(state.err, errno, "unexpected error");
+                neo4j_perror(state.err, errno, "Unexpected error");
                 goto cleanup;
             }
             break;
         case 'p':
             if (neo4j_config_set_password(state.config, optarg))
             {
-                neo4j_perror(state.err, errno, "unexpected error");
+                neo4j_perror(state.err, errno, "Unexpected error");
                 goto cleanup;
             }
+            password_set = true;
             break;
         case 'P':
-            if (tty == NULL)
-            {
-                fprintf(state.err,
-                        "Cannot prompt for a password without a tty\n");
-                goto cleanup;
-            }
             state.password_prompt = true;
             break;
         case KNOWN_HOSTS_OPT:
             if (neo4j_config_set_known_hosts_file(state.config, optarg))
             {
-                neo4j_perror(state.err, errno, "unexpected error");
+                neo4j_perror(state.err, errno, "Unexpected error");
                 goto cleanup;
             }
             break;
         case NO_KNOWN_HOSTS_OPT:
             if (neo4j_config_set_trust_known_hosts(state.config, false))
             {
-                neo4j_perror(state.err, errno, "unexpected error");
+                neo4j_perror(state.err, errno, "Unexpected error");
                 goto cleanup;
             }
             break;
@@ -355,6 +394,13 @@ int main(int argc, char *argv[])
         goto cleanup;
     }
 
+    if (argc == 0 && password_set)
+    {
+        fprintf(stderr, "--password/-p can only be used when a URL or "
+                "host to connect to is also supplied.\n");
+        goto cleanup;
+    }
+
     uint8_t logger_flags = 0;
     if (log_level < NEO4J_LOG_DEBUG)
     {
@@ -363,50 +409,49 @@ int main(int argc, char *argv[])
     provider = neo4j_std_logger_provider(state.err, log_level, logger_flags);
     if (provider == NULL)
     {
-        neo4j_perror(state.err, errno, "unexpected error");
+        neo4j_perror(state.err, errno, "Unexpected error");
         goto cleanup;
     }
 
     neo4j_config_set_logger_provider(state.config, provider);
 
-    if (state.interactive)
-    {
-        state.password_prompt = true;
-    }
-
     if (tty != NULL)
     {
         neo4j_config_set_unverified_host_callback(state.config,
                 host_verification, &state);
 
-        if (state.password_prompt)
+        if (state.interactive)
         {
-            neo4j_config_set_authentication_reattempt_callback(state.config,
-                    auth_reattempt, &state);
+            state.password_prompt = true;
         }
     }
+    else if (state.password_prompt)
+    {
+        fprintf(state.err,
+                "Cannot prompt for a password in non-interactive mode\n");
+        goto cleanup;
+    }
 
-    if (argc >= 1 && db_connect(&state, argv[0], (argc > 1)? argv[1] : NULL))
+    if (argc >= 1 && db_connect(&state, cypher_input_position_zero,
+                argv[0], (argc > 1)? argv[1] : NULL))
     {
         goto cleanup;
     }
 
     // remove any password from the config
-    if (neo4j_config_set_password(state.config, NULL))
-    {
-        // can't fail
-    }
+    ignore_unused_result(neo4j_config_set_password(state.config, NULL));
 
     if (signal(SIGINT, interrupt_handler) == SIG_ERR)
     {
-        perror("unexpected error");
+        perror("Unexpected error");
         goto cleanup;
     }
 
     if (state.interactive)
     {
         state.render = render_results_table;
-        state.render_flags = NEO4J_RENDER_SHOW_NULLS;
+        neo4j_config_set_render_nulls(state.config, true);
+        state.show_timing = true;
         state.infile = "<interactive>";
         state.source_depth = 1;
         if (interact(&state))
@@ -419,7 +464,8 @@ int main(int argc, char *argv[])
         state.render = render_results_csv;
         for (unsigned int i = 0; i < nio_handlers; ++i)
         {
-            if (io_handlers[i].handle(&state, io_handlers[i].arg))
+            if (io_handlers[i].handle(&state, cypher_input_position_zero,
+                    io_handlers[i].arg))
             {
                 goto cleanup;
             }
@@ -430,7 +476,7 @@ int main(int argc, char *argv[])
         state.render = render_results_csv;
         state.infile = "<stdin>";
         state.source_depth = 1;
-        if (batch(&state, state.in))
+        if (batch(&state, cypher_input_position_zero, state.in))
         {
             goto cleanup;
         }
@@ -459,7 +505,8 @@ cleanup:
 
 int add_io_handler(struct io_handler *io_handlers, unsigned int *nio_handlers,
         const char *arg, bool is_output,
-        int (*handler)(shell_state_t *state, const char *arg))
+        int (*handler)(shell_state_t *state, struct cypher_input_position pos,
+            const char *arg))
 {
     if (*nio_handlers >= NEO4J_MAX_IO_ARGS)
     {
@@ -477,8 +524,8 @@ int add_io_handler(struct io_handler *io_handlers, unsigned int *nio_handlers,
 
 void interrupt_handler(int signal)
 {
-    if (state.session != NULL && neo4j_reset_session(state.session))
+    if (state.connection != NULL && neo4j_reset(state.connection))
     {
-        neo4j_perror(stderr, errno, "reset failed");
+        neo4j_perror(stderr, errno, "Reset failed");
     }
 }
