@@ -17,6 +17,7 @@
 #include "../../config.h"
 #include "util.h"
 #include <assert.h>
+#include <ctype.h>
 #include <string.h>
 
 
@@ -32,7 +33,7 @@ char *strncpy_alloc(char **dest, size_t *cap, const char *s, size_t n)
         *dest = updated;
         *cap = n+1;
     }
-    memcpy(*dest, s, n+1);
+    strncpy(*dest, s, n+1);
     (*dest)[n] = '\0';
     return *dest;
 }
