@@ -9,13 +9,14 @@ TCase* dotdir_tcase(void);
 TCase* error_handling_tcase(void);
 TCase* logging_tcase(void);
 TCase* memory_tcase(void);
+TCase* fit_column_widths_tcase(void);
 TCase* render_plan_tcase(void);
 TCase* render_results_tcase(void);
 TCase* result_stream_tcase(void);
 TCase* ring_buffer_tcase(void);
 TCase* serialization_tcase(void);
-TCase* session_tcase(void);
 TCase* tofu_tcase(void);
+TCase* u8_tcase(void);
 TCase* uri_tcase(void);
 TCase* util_tcase(void);
 TCase* values_tcase(void);
@@ -33,13 +34,14 @@ Suite *libneo4j_client_suite(void)
     suite_add_tcase(s, error_handling_tcase());
     suite_add_tcase(s, logging_tcase());
     suite_add_tcase(s, memory_tcase());
+    suite_add_tcase(s, fit_column_widths_tcase());
     suite_add_tcase(s, render_plan_tcase());
     suite_add_tcase(s, render_results_tcase());
     suite_add_tcase(s, result_stream_tcase());
     suite_add_tcase(s, ring_buffer_tcase());
     suite_add_tcase(s, serialization_tcase());
-    suite_add_tcase(s, session_tcase());
     suite_add_tcase(s, tofu_tcase());
+    suite_add_tcase(s, u8_tcase());
     suite_add_tcase(s, uri_tcase());
     suite_add_tcase(s, util_tcase());
     suite_add_tcase(s, values_tcase());
