@@ -105,7 +105,7 @@ START_TEST (render_simple_plan)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 73, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 73, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
@@ -114,9 +114,9 @@ START_TEST (render_simple_plan)
  "+------------------+----------------+-------------+---------------------+\n"
  "| Operator         | Estimated Rows | Identifiers | Other               |\n"
  "+------------------+----------------+-------------+---------------------+\n"
- "| +NodeByLabelScan |             10 | n           | :Person             |\n"
+ "| *NodeByLabelScan |             10 | n           | :Person             |\n"
  "| |                +----------------+-------------+---------------------+\n"
- "| +ProduceResults  |              5 | n, m        |                     |\n"
+ "| *ProduceResults  |              5 | n, m        |                     |\n"
  "+------------------+----------------+-------------+---------------------+\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
@@ -142,7 +142,7 @@ START_TEST (render_branched_plan)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 73, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 73, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
@@ -151,12 +151,12 @@ START_TEST (render_branched_plan)
  "+--------------------+----------------+----------------+----------------+\n"
  "| Operator           | Estimated Rows | Identifiers    | Other          |\n"
  "+--------------------+----------------+----------------+----------------+\n"
- "| +NodeByLabelScan   |             10 | n              | :Person        |\n"
+ "| *NodeByLabelScan   |             10 | n              | :Person        |\n"
  "| |                  +----------------+----------------+----------------+\n"
- "| | +NodeByLabelScan |             10 | n, m, l, k, j, | :City; n.age > |\n"
+ "| | *NodeByLabelScan |             10 | n, m, l, k, j, | :City; n.age > |\n"
  "| | |                |                |  i, h, g       |  { AUTOINT0 }  |\n"
  "| |/                 +----------------+----------------+----------------+\n"
- "| +ProduceResults    |              5 | n, m, l        |                |\n"
+ "| *ProduceResults    |              5 | n, m, l        |                |\n"
  "+--------------------+----------------+----------------+----------------+\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
@@ -186,7 +186,7 @@ START_TEST (render_multi_branched_plan)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 73, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 73, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
@@ -195,15 +195,15 @@ START_TEST (render_multi_branched_plan)
  "+--------------------+----------------+----------------+----------------+\n"
  "| Operator           | Estimated Rows | Identifiers    | Other          |\n"
  "+--------------------+----------------+----------------+----------------+\n"
- "| +NodeByLabelScan   |             10 | n, m, l, k, j, | :Person        |\n"
+ "| *NodeByLabelScan   |             10 | n, m, l, k, j, | :Person        |\n"
  "| |                  |                |  i, h, g       |                |\n"
  "| |                  +----------------+----------------+----------------+\n"
- "| | +NodeByLabelScan |             10 | m              | :City; n.age > |\n"
+ "| | *NodeByLabelScan |             10 | m              | :City; n.age > |\n"
  "| | |                |                |                |  { AUTOINT0 }  |\n"
  "| |/                 +----------------+----------------+----------------+\n"
- "| | +NodeByLabelScan |             10 | l              | :Person        |\n"
+ "| | *NodeByLabelScan |             10 | l              | :Person        |\n"
  "| |/                 +----------------+----------------+----------------+\n"
- "| +ProduceResults    |              5 | n, m           |                |\n"
+ "| *ProduceResults    |              5 | n, m           |                |\n"
  "+--------------------+----------------+----------------+----------------+\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
@@ -231,7 +231,7 @@ START_TEST (render_deep_branched_plan)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 73, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 73, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
@@ -240,15 +240,15 @@ START_TEST (render_deep_branched_plan)
  "+----------------------+----------------+-------------+-----------------+\n"
  "| Operator             | Estimated Rows | Identifiers | Other           |\n"
  "+----------------------+----------------+-------------+-----------------+\n"
- "| +NodeByLabelScan     |             10 | n           | :Person         |\n"
+ "| *NodeByLabelScan     |             10 | n           | :Person         |\n"
  "| |                    +----------------+-------------+-----------------+\n"
- "| | +NodeByLabelScan   |             10 | l           | :Person         |\n"
+ "| | *NodeByLabelScan   |             10 | l           | :Person         |\n"
  "| | |                  +----------------+-------------+-----------------+\n"
- "| | | +NodeByLabelScan |             10 | k           | :Person         |\n"
+ "| | | *NodeByLabelScan |             10 | k           | :Person         |\n"
  "| | |/                 +----------------+-------------+-----------------+\n"
- "| | +NodeByLabelScan   |             10 | m           | :Person         |\n"
+ "| | *NodeByLabelScan   |             10 | m           | :Person         |\n"
  "| |/                   +----------------+-------------+-----------------+\n"
- "| +ProduceResults      |              5 | n, m        |                 |\n"
+ "| *ProduceResults      |              5 | n, m        |                 |\n"
  "+----------------------+----------------+-------------+-----------------+\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
@@ -275,7 +275,7 @@ START_TEST (render_narrow_plan)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 61, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 61, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
@@ -284,17 +284,17 @@ START_TEST (render_narrow_plan)
  "+--------------------+----------------+-------------+-------+\n"
  "| Operator           | Estimated Rows | Identifiers | Other |\n"
  "+--------------------+----------------+-------------+-------+\n"
- "| +NodeByLabelScan   |             10 | n, m, l, k, | :Pers |\n"
+ "| *NodeByLabelScan   |             10 | n, m, l, k, | :Pers |\n"
  "| |                  |                |  j, i, h, g | on    |\n"
  "| |                  +----------------+-------------+-------+\n"
- "| | +NodeByLabelScan |             10 | m           | :City |\n"
+ "| | *NodeByLabelScan |             10 | m           | :City |\n"
  "| | |                |                |             | ; n.a |\n"
  "| | |                |                |             | ge >  |\n"
  "| | |                |                |             | { AUT |\n"
  "| | |                |                |             | OINT0 |\n"
  "| | |                |                |             |  }    |\n"
  "| |/                 +----------------+-------------+-------+\n"
- "| +ProduceResults    |              5 | n, m, l     |       |\n"
+ "| *ProduceResults    |              5 | n, m, l     |       |\n"
  "+--------------------+----------------+-------------+-------+\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
@@ -314,22 +314,22 @@ START_TEST (render_undersized_plan)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 60, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 60, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
     const char *expect = "\n"
 //1       10        20        30        40        50        60        70
- "+--------------------+----------------+-------------+=\n"
+ "+--------------------+----------------+-------------+-\n"
  "| Operator           | Estimated Rows | Identifiers |=\n"
- "+--------------------+----------------+-------------+=\n"
- "| +NodeByLabelScan   |             10 | n           |=\n"
- "| |                  +----------------+-------------+=\n"
- "| | +NodeByLabelScan |             10 | m           |=\n"
- "| |/                 +----------------+-------------+=\n"
- "| +ProduceResults    |              5 | n, m, l, k, |=\n"
+ "+--------------------+----------------+-------------+-\n"
+ "| *NodeByLabelScan   |             10 | n           |=\n"
+ "| |                  +----------------+-------------+-\n"
+ "| | *NodeByLabelScan |             10 | m           |=\n"
+ "| |/                 +----------------+-------------+-\n"
+ "| *ProduceResults    |              5 | n, m, l, k, |=\n"
  "|                    |                |  j, i, h, g |=\n"
- "+--------------------+----------------+-------------+=\n";
+ "+--------------------+----------------+-------------+-\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
 END_TEST
@@ -348,21 +348,21 @@ START_TEST (render_very_undersized_plan)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 50, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 50, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
     const char *expect = "\n"
 //1       10        20        30        40        50        60        70
- "+--------------------+----------------+=\n"
+ "+--------------------+----------------+-\n"
  "| Operator           | Estimated Rows |=\n"
- "+--------------------+----------------+=\n"
- "| +NodeByLabelScan   |             10 |=\n"
- "| |                  +----------------+=\n"
- "| | +NodeByLabelScan |             10 |=\n"
- "| |/                 +----------------+=\n"
- "| +ProduceResults    |              5 |=\n"
- "+--------------------+----------------+=\n";
+ "+--------------------+----------------+-\n"
+ "| *NodeByLabelScan   |             10 |=\n"
+ "| |                  +----------------+-\n"
+ "| | *NodeByLabelScan |             10 |=\n"
+ "| |/                 +----------------+-\n"
+ "| *ProduceResults    |              5 |=\n"
+ "+--------------------+----------------+-\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
 END_TEST
@@ -381,21 +381,21 @@ START_TEST (render_vvery_undersized_plan)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 38, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 38, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
     const char *expect = "\n"
 //1       10        20        30        40        50        60        70
- "+--------------------+=\n"
+ "+--------------------+-\n"
  "| Operator           |=\n"
- "+--------------------+=\n"
- "| +NodeByLabelScan   |=\n"
- "| |                  +=\n"
- "| | +NodeByLabelScan |=\n"
- "| |/                 +=\n"
- "| +ProduceResults    |=\n"
- "+--------------------+=\n";
+ "+--------------------+-\n"
+ "| *NodeByLabelScan   |=\n"
+ "| |                  +-\n"
+ "| | *NodeByLabelScan |=\n"
+ "| |/                 +-\n"
+ "| *ProduceResults    |=\n"
+ "+--------------------+-\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
 END_TEST
@@ -414,21 +414,21 @@ START_TEST (render_vvvery_undersized_plan)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 21, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 21, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
     const char *expect = "\n"
 //1       10        20        30        40        50        60        70
- "+=\n"
+ "+-\n"
  "|=\n"
- "+=\n"
+ "+-\n"
  "|=\n"
  "|=\n"
  "|=\n"
  "|=\n"
  "|=\n"
- "+=\n";
+ "+-\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
 END_TEST
@@ -445,7 +445,7 @@ START_TEST (render_simple_profile)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 78, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 78, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
@@ -454,9 +454,9 @@ START_TEST (render_simple_profile)
  "+------------------+----------------+------+---------+-------------+---------+\n"
  "| Operator         | Estimated Rows | Rows | DB Hits | Identifiers | Other   |\n"
  "+------------------+----------------+------+---------+-------------+---------+\n"
- "| +NodeByLabelScan |             10 |    5 |      42 | n           | :Person |\n"
+ "| *NodeByLabelScan |             10 |    5 |      42 | n           | :Person |\n"
  "| |                +----------------+------+---------+-------------+---------+\n"
- "| +ProduceResults  |              5 |    8 |     935 | n, m        |         |\n"
+ "| *ProduceResults  |              5 |    8 |     935 | n, m        |         |\n"
  "+------------------+----------------+------+---------+-------------+---------+\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
@@ -474,19 +474,19 @@ START_TEST (render_narrow_profile)
 
     fputc('\n', memstream);
 
-    int result = neo4j_render_plan_table(memstream, plan, 60, 0);
+    int result = neo4j_render_plan_table(memstream, plan, 60, NEO4J_RENDER_ASCII);
     ck_assert(result == 0);
     fflush(memstream);
 
     const char *expect = "\n"
 //1       10        20        30        40        50        60        70
- "+------------------+----------------+------+---------+=\n"
+ "+------------------+----------------+------+---------+-\n"
  "| Operator         | Estimated Rows | Rows | DB Hits |=\n"
- "+------------------+----------------+------+---------+=\n"
- "| +NodeByLabelScan |             10 |    5 |      42 |=\n"
- "| |                +----------------+------+---------+=\n"
- "| +ProduceResults  |              5 |    8 |     935 |=\n"
- "+------------------+----------------+------+---------+=\n";
+ "+------------------+----------------+------+---------+-\n"
+ "| *NodeByLabelScan |             10 |    5 |      42 |=\n"
+ "| |                +----------------+------+---------+-\n"
+ "| *ProduceResults  |              5 |    8 |     935 |=\n"
+ "+------------------+----------------+------+---------+-\n";
     ck_assert_str_eq(memstream_buffer, expect);
 }
 END_TEST
